@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2005-2010 Bastian Kleineidam
+# Copyright (C) 2005-2011 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@ import locale
 import stat
 import fnmatch
 import mimetypes
+import tempfile
+from distutils.spawn import find_executable
 
 
 def write_file (filename, content, backup=False, callback=None):
@@ -63,7 +65,8 @@ def has_module (name):
     try:
         exec "import %s" % name
         return True
-    except ImportError:
+    except (OSError, ImportError):
+        # some modules (for example HTMLtidy) raise OSError
         return False
 
 
@@ -159,6 +162,7 @@ else:
     FSCODING = "utf-8"
 
 def pathencode (path):
+    """Encode a path string with the platform file system encoding."""
     if isinstance(path, unicode) and not os.path.supports_unicode_filenames:
         path = path.encode(FSCODING, "replace")
     return path
@@ -178,11 +182,17 @@ def has_changed (filename):
 
 
 mimedb = mimetypes.MimeTypes(strict=False)
+# For Opera bookmark files (opera6.adr)
+mimedb.add_type('text/plain', '.adr', strict=False)
+# To recognize PHP files as HTML with content check.
+mimedb.add_type('text/plain', '.php', strict=False)
+
 
 # if file extension lookup was unsuccessful, look at the content
 PARSE_CONTENTS = {
     "text/html": re.compile(r'^(?i)<(!DOCTYPE html|html|head|title)'),
     "text/plain+opera": re.compile(r'^Opera Hotlist'),
+    "text/plain+chromium": re.compile(r'^{\s*"checksum":'),
     "text/plain+linkchecker": re.compile(r'(?i)^# LinkChecker URL list'),
 }
 
@@ -190,8 +200,11 @@ def guess_mimetype (filename, read=None):
     """Return MIME type of file, or 'application/octet-stream' if it could
     not be determined."""
     mime, encoding = mimedb.guess_type(filename, strict=False)
+    # Special case for Google Chrome Bookmark files.
+    if not mime and os.path.basename(filename) == 'Bookmarks':
+        mime = 'text/plain'
     # Mime type text/plain can be differentiated further with content reading.
-    if (mime == "text/plain" or not mime) and read is not None:
+    if mime == "text/plain" and read is not None:
         # try to read some content and do a poor man's file(1)
         # XXX replace with file(1) on Unix systems
         try:
@@ -207,3 +220,10 @@ def guess_mimetype (filename, read=None):
         # split off not needed extension info
         mime = mime.split(';')[0]
     return mime.strip().lower()
+
+
+def get_temp_file (**kwargs):
+    """Return tuple (open file object, filename) pointing to a temporary
+    file."""
+    fd, filename = tempfile.mkstemp(**kwargs)
+    return os.fdopen(fd), filename
